using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;
using System.Diagnostics;
using System.Threading;
using System.Collections;

namespace HelpMate
{
    public partial class mainfrm : Form
    {
        private string[] _comportList;
        private string _data;
        private string _temperature;
        private delegate void ConfigurationDelegate();
        private bool _continouse = false;
        private bool IsConnected = false;
        private Thread SearchDevice;
        private string[] _DeviceList=new string[111];
        private bool _IsSearch = false;
        private int _deviceCounter = 0;

        private int _lookingID = 0;

        public mainfrm()
        {
            InitializeComponent();
        }

        public void SetConfiguration()
        {

            // InvokeRequired compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.


            if (this.InvokeRequired)
            {
                ConfigurationDelegate d = new ConfigurationDelegate(SetConfiguration);
                this.Invoke(d, new object[] { });
            }
            else
            {
                // toolStripStatusLabel_CBVersion.Text = _iTahrd.BoardVersion;

                label_Data.Text = _data;
                label_Temperature.Text = _temperature;
                richTextBox_DeviceList.Clear();

                if (_IsSearch)
                {
                    richTextBox_DeviceList.AppendText("Searching Sensors Address 0x" + _lookingID.ToString("x") + "\n");
                }

                for (int i = 0; i < _deviceCounter;i++ )
                {
                   

                    richTextBox_DeviceList.AppendText(_DeviceList[i] + "\n");
                }


            }



        }

        private void Form1_Load(object sender, EventArgs e)
        {

            RegistryKey keyCom = Registry.LocalMachine.OpenSubKey("Hardware\\DeviceMap\\SerialComm");
            if (keyCom != null)
            {
                string[] sSubKeys = keyCom.GetValueNames();

                _comportList = new string[keyCom.ValueCount];

                int i = 0;
                foreach (string sName in sSubKeys)
                {
                    _comportList[i] = (string)keyCom.GetValue(sName);
                    i++;

                }
            }

            comboBox_SerialPort.DataSource = _comportList;
        }

        private void serialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            byte[] header = new byte[2];

            header[0] = System.Convert.ToByte(serialPort.ReadByte());
            header[1] = System.Convert.ToByte(serialPort.ReadByte());

            byte address =System.Convert.ToByte( serialPort.ReadByte());

            byte length =System.Convert.ToByte( serialPort.ReadByte());

            byte cmd =System.Convert.ToByte( serialPort.ReadByte());

            switch (length)
            {
                case 1:
                    {
                        byte flag = System.Convert.ToByte(serialPort.ReadByte());
                        byte sum = System.Convert.ToByte(serialPort.ReadByte());

                        byte calsum = 0x00;

                        calsum = header[0];
                        calsum += header[1];
                        calsum += address;
                        calsum += length;
                        calsum += cmd;
                        calsum += flag;


                        if (flag ==1)
                        {


                            MessageBox.Show("Set Address Sucessfully!");

                        }

                        break;
                    }
                case 2:
                    {
                       
                      
                        byte high = System.Convert.ToByte(serialPort.ReadByte());
                        byte low = System.Convert.ToByte(serialPort.ReadByte());
                        byte sum = System.Convert.ToByte(serialPort.ReadByte());

                        byte calsum = 0x00;

                        calsum = header[0];
                        calsum += header[1];
                        calsum += address;
                        calsum += length;
                        calsum += cmd;
                        calsum += high;
                        calsum += low;

                        if (calsum == sum)
                        {

                          

                        }

                        int result = high * 255 + low;

                        switch (cmd)
                        {
                            case 0x02:
                                {
                                    _data = result .ToString();
                                    break;
                                }
                            case 0x03:
                                {
                                    double temp = result / 10;
                                    _temperature = temp.ToString();


                                    if (_IsSearch)
                                    {
                                        int id = _deviceCounter + 1;
                                        _DeviceList[_deviceCounter] = "ID:" + id.ToString()+ " Address:" + "0x"+address.ToString("x");

                                        _deviceCounter++;
                                    }

                                    break;
                                }

                        }

                    
                        SetConfiguration();

                        if (_continouse)
                        {
                            int delay = (int) numericUpDown_ReadInterval.Value;
                            TriggerReading();
                            Thread.Sleep(delay);
                            ReadDistance();
                        }

                        break;
                    }
            }


        }

        private void btn_SetAddress_Click(object sender, EventArgs e)
        {
            byte[] _cmd = new byte[7];
            _cmd[0] = 0x55;
            _cmd[1] = 0xaa;
            _cmd[2] = 0xab;
            _cmd[3] = 0x01;
            _cmd[4] = 0x55;
            _cmd[5] = System.Convert.ToByte(numericUpDown_Address.Value);

            byte sum = 0x00;

            for (int i = 0; i < _cmd.Length; i++)
            {
                sum += _cmd[i];
            }

            _cmd[6] = sum;

            if (IsConnected)
            {
               

                serialPort.Write(_cmd, 0, _cmd.Length);
            }

        }

        private void btn_Connect_Click(object sender, EventArgs e)
        {
            try
            {
                if (serialPort.IsOpen)
                {
                    serialPort.Close();
                    btn_Connect.Text = "Connect";
                    IsConnected = false;

                    btn_ReadTemperature.Enabled = false;
                    btn_ReadDistance.Enabled = false;
                    btn_SearchDevice.Enabled = false;
                    btn_SetAddress.Enabled = false;
                }
                else
                {
                    serialPort.PortName = comboBox_SerialPort.Text;
                    serialPort.BaudRate = System.Convert.ToInt32(comboBox_BaudRate.Text);
                    serialPort.Open();

                    IsConnected = true;

                    btn_ReadTemperature.Enabled = true;
                    btn_ReadDistance.Enabled = true;
                    btn_SearchDevice.Enabled = true;
                    btn_SetAddress.Enabled = true;

                    btn_Connect.Text = "Disconnect";
                }
            }
            catch
            {
                MessageBox.Show("Can not open Serial Port"+comboBox_SerialPort.Text+"!");
            }
        }

        private void btn_ReadDistance_Click(object sender, EventArgs e)
        {
            if (btn_ReadDistance.Text == "Read Distance")
            {
                TriggerReading();

                Thread.Sleep(85);

                ReadDistance();

                if (checkBox_Continous.Checked)
                {

                    btn_ReadDistance.Text = "Stop";

                    btn_ReadTemperature.Enabled = false;
                }
            }
            else
            {
                _continouse = false;
                checkBox_Continous.Checked = false;
                btn_ReadDistance.Text = "Read Distance";

                btn_ReadTemperature.Enabled = true;
            }
        }



        private void btn_ReadTemperature_Click(object sender, EventArgs e)
        {

            if (btn_ReadTemperature.Text == "ReadTemperature")
            {
                ReadTemperature(System.Convert.ToByte(numericUpDown_Sensor.Value));

               // btn_ReadTemperature.Text = "ֹͣȡ";
            }
            else
            {
               // _continouse = false;
               // checkBox_Continous.Checked = false;
               // btn_ReadTemperature.Text = "ȡ¶";
            }

        }

        private void TriggerReading()
        {
            byte[] _cmd = new byte[6];
            _cmd[0] = 0x55;
            _cmd[1] = 0xaa;
            _cmd[2] = System.Convert.ToByte(numericUpDown_Sensor.Value);
            _cmd[3] = 0x00;
            _cmd[4] = 0x01;


            byte sum = 0x00;

            for (int i = 0; i < _cmd.Length; i++)
            {
                sum += _cmd[i];
            }

            _cmd[5] = sum;

            if (IsConnected)
            {

                serialPort.Write(_cmd, 0, _cmd.Length);
            }
        }

        private void ReadDistance()
        {
            byte[] _cmd = new byte[6];
            _cmd[0] = 0x55;
            _cmd[1] = 0xaa;
            _cmd[2] = System.Convert.ToByte(numericUpDown_Sensor.Value);
            _cmd[3] = 0x00;
            _cmd[4] = 0x02;


            byte sum = 0x00;

            for (int i = 0; i < _cmd.Length; i++)
            {
                sum += _cmd[i];
            }

            _cmd[5] = sum;

            if (IsConnected)
            {

                serialPort.Write(_cmd, 0, _cmd.Length);
            }
        }



        private void ReadTemperature(byte address)
        {
            byte[] _cmd = new byte[6];
            _cmd[0] = 0x55;
            _cmd[1] = 0xaa;
            _cmd[2] = address;
            _cmd[3] = 0x00;
            _cmd[4] = 0x03;


            byte sum = 0x00;

            for (int i = 0; i < _cmd.Length; i++)
            {
                sum += _cmd[i];
            }

            _cmd[5] = sum;

            if (IsConnected)
            {
                serialPort.Write(_cmd, 0, _cmd.Length);
            }

        }


        private void checkBox_Continous_CheckedChanged(object sender, EventArgs e)
        {
            _continouse = checkBox_Continous.Checked;
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            System.Diagnostics.Process[] myProcesses = System.Diagnostics.Process.GetProcesses();



            foreach (Process process in System.Diagnostics.Process.GetProcesses())
            {
                if (process.ProcessName.ToUpper().Equals("URMV4HELPMATE"))
                    process.Kill();
            }

        
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("IExplore", linkLabel1.Text);
        }

        private void btn_SearchDevice_Click(object sender, EventArgs e)
        {


            if (btn_SearchDevice.Text == "Search Sensors")
                {
                    
                    btn_ReadDistance.Enabled = false;
                    btn_ReadTemperature.Enabled = false;

                    btn_SearchDevice.Text = "Stope";

                    

                    SearchDevice = new Thread(new ThreadStart(LookupDevice));

                    SearchDevice.Start();

                    _IsSearch = true;
                    _deviceCounter = 0;
                }
                else
                {
                    SearchDevice.Abort();
                    btn_SearchDevice.Text = "Search Sensors";
                    btn_ReadDistance.Enabled = true;
                    btn_ReadTemperature.Enabled = true;

                    _IsSearch = false;

                }
            

        }


        private void LookupDevice()
        {
            _lookingID = 17;

            for(int i=17;i<=128;i++)
            {

                byte address = (byte)i;

                ReadTemperature(address);

                Thread.Sleep(200);
                _lookingID = i;
                SetConfiguration();

            }

            btn_SearchDevice.Text = "Search Sensors";
            btn_ReadDistance.Enabled = true;
            btn_ReadTemperature.Enabled = true;

            _IsSearch = false;
        }
    }
}